# This script contains the function to overlap continous featues with annotations
#
# Author: Judith Zaugg
###############################################################################

overlapFeaturesAnnotations=function(
		annoTab=NULL,featureTab,
		align=c('start','end','full'),
		up=500,down=500,
		datafolder=NULL,
		projectfolder=getwd(),
		featureNames = NULL,
		logFile=TRUE,
		method=c(shorth,mean)){
	require(GenomicRanges)
	require(genefilter)
	
# first do all the testing and converting
# featuresTab
	if(class(featureTab)=='data.frame'){
		stopifnot(!is.null(featureTab$first),!is.null(featureTab$last),!is.null(featureTab$seqName),!is.null(featureTab$id))
		featureTabGR=GRanges(
				seqnames=featureTab$seqName,
				strand=featureTab$strand,
				ranges=IRanges(start=featureTab$first,end=featureTab$last,name=featureTab$id))
		featureData=featureTab
		featureTab=featureTabGR
		cat('converted featureTab into GRanges object\n')
		
	} else {
		stopifnot(class(featureTab)=='GRanges')
		stopifnot(!is.null(elementMetadata(featureTab)))
		featureData=as.data.frame(elementMetadata(featureTab))
		featureData$id=names(featureTab)
		
	}
# remove the last slashes from the folders
	datafolder=gsub('/$','',datafolder)
	projectfolder=gsub('/$','',projectfolder)
# projectfolder
	createFolderIfNotExist(projectfolder)
# datafolder
	if(!is.null(datafolder)){
		createFolderIfNotExist(datafolder)
	}else{
		datafolder=paste(projectfolder,'data',sep='/')
		createFolderIfNotExist(datafolder)
	}
# check whether a logfile exists and create one, if there is none
	if(logFile){
		logfile=paste(projectfolder,'AGP.log',sep='/')
		if(!file.exists(logfile)){	
			cat('#this is the logfile for the project in project folder',file.path(projectfolder),'\n\n',file=logfile)		
			cat(timestamp(),'\n',file=logfile,append=T)
			cat('created logfile',logfile,'\n')
		}
	}
# annoTab
	
	annoTab=loadAnnoTab(projectfolder,annoTabID=annoTab,FALSE)
	annofile=paste(projectfolder,'annoTab.rda',sep='/')
#	if(!is.null(annoTab)&!file.exists(annofile)){
#		cat('create a new',annofile,'\n')
#		rownames(elementMetadata(annoTab))=names(annoTab)
#		if(logFile)
#			cat('file created:',annofile,'\n',file=logfile,append=T)
#		cat('file created:',annofile,'\n')
#		
#	} else if(file.exists(annofile)){
#		cat(annofile, 'exists already\n')
#		if(!is.null(annoTab))
#			cat('WARNING: supplied annoTab argument ignored',annofile,'used instead\n')
#		load(annofile)
#		if(logFile)
#			cat('add to:',annofile,'\n',file=logfile, append=T)
#		cat('add to:',annofile,'\n')
#	}
	if(class(annoTab)=='data.frame'){
		stopifnot(!is.null(annoTab$first),!is.null(annoTab$strand),!is.null(annoTab$last),!is.null(annoTab$seqName))
		cat('create GRanges object from annoTab \n')
		annoGR=GRanges(
				seqnames = annoTab$seqName, 
				strand=annoTab$strand, 
				ranges = IRanges(
						start=annoTab$first, 
						end = annoTab$last,
						names=annoTab$id))
	} else {
		stopifnot(class(annoTab)=="GRanges")
		cat('create data.frame GRanges object from annoTab \n')
		annoGR=annoTab
		annoTab=elementMetadata(annoGR)
		annoTab$id=names(annoGR)
		annoTab$first=start(annoGR)
		annoTab$last=end(annoGR)
		annoTab$strand=as.vector(strand(annoGR))
		annoTab$seqName=as.vector(seqnames(annoGR))
		
	}
	
# define the borders of the alignements
	plusstrand=as.character(as.vector(strand(annoGR)))=='+'
	if(align=='start'){
		start(annoGR)[plusstrand]=start(annoGR)[plusstrand]-up
		end(annoGR)[plusstrand]=start(annoGR)[plusstrand]+down
		end(annoGR)[!plusstrand]=end(annoGR)[!plusstrand]+up
		start(annoGR)[!plusstrand]=end(annoGR)[!plusstrand]-down
	} else if(align=='end'){
		start(annoGR)[plusstrand]=end(annoGR)[plusstrand]-up
		end(annoGR)[plusstrand]=end(annoGR)[plusstrand]+down
		end(annoGR)[!plusstrand]=start(annoGR)[!plusstrand]+up
		start(annoGR)[!plusstrand]=start(annoGR)[!plusstrand]-down
	}
# find overlaps
	mM=(matchMatrix(findOverlaps(subject=annoGR,query=featureTab,type='any',select='all')))
# add all numeric data values and keep record of them in the log file	
	mM=cbind(mM,id=as.numeric(names(annoGR)[mM[,'subject']]))
	mM_L=split(mM[,'query'],mM[,'id'])
	
# define the columns to add to the annoGR: take all numeric cols from featureData except the coordinates
	cols=colnames(featureData)[sapply(featureData,class)=='numeric']
	cols=cols[!cols%in%c('id','first','last','seqName')]
	cat('align',cols,'at',align,'\n',file=logfile,append=T)
	for(i in 1:length(cols)){
		c=cols[i]
		means=sapply(mM_L,function(x){
					method(featureData[[c]][x],na.rm=T)	
				})
		c=paste(featureNames,c,align,sep='.')
		if(!is.null(elementMetadata(annoGR)[[c]]))
			c=paste(c,1,sep='_')
		elementMetadata(annoGR)[[c]]=means[match(names(annoGR),names(mM_L))]
		if(logFile)
			cat('added:',c,'to',annofile,'\n',file=logfile,append=T)
		cat('added:',c,'to',annofile,'\n')
		
	}
	annoTab2=as.data.frame(elementMetadata(annoGR))
	annoTab=cbind(annoTab,annoTab2)
	save(annoTab,file=annofile)
	cat('saved:',annofile,'\n',file=logfile,append=T)
	return(annofile)
}

