% 
% This is the annotation for the overlapFeaturesAnnotations function
% update it after the function is written

\name{overlapFeaturesAnnotations}
\alias{overlapFeaturesAnnotations}
\title{overlapFeaturesAnnotations}
\description{This function is to overlap a features (continuous values) with a specific region, defined by an annotation data.frame by averaging the feature signal over the annotation region. It returns the filename where the annotation table overlaid with the provided feature were save. In addition, it adds information to the logfile in the project folder.}
\usage{
overlapFeaturesAnnotations( 
	annoTab=NULL, 
	featureTab, 
	align=c('start','end','full'),
	up=500,down=500,
	datafolder=NULL,
	projectfolder=getwd(),
	featureNames = NULL)}
\arguments{
	\item{annoTab}{If provided, must be of class \code{data.frame} or \code{GRange} and contain at least the following slots: \code{id, tx_ID, stand, seqName, first, last} denoting the genomic coordinates of the annotations. If not provided, a file called annoTab.rda must be present in the project directory in a data folder. Will be created by calling this function. If an annoTab file already exists in the project folder, this file will be used and a warning will be issued.}
	\item{featureTab}{Must be of class \code{data.frame} of \code{GRange} and contain at least the following slots: \code{id, tx_ID, seqName, first, last, value} denoting the genomic coordinates and the value at each position.}
	\item{align}{Determines the middle of the region over which the feature value will be averaged. Possible values are start, end, full. If \code{start} or \code{end} are chosen, the boundaries are determined by \code{up, down}. If \code{full} is chosen the feature value is averaged across the full region of the annoTab.}
	\item{up}{Determines the upstream end of the region.}
	\item{down}{Determines the downstream end of the region.}
	\item{datafolder}{The relative path to the folder in which the data set will be saved to. The default is a folder called 'data' in the working directory.}
	\item{featureNames}{The name used to identify and save the data set.}
	\item{projectfolder}{The name of the folder where the project is carried out. Default is the working directory. This is used to update the logfile.}
}

\details{
}

\value{The path to the annoTab in the project folder.}
\author{Judith Zaugg}

\examples{
#source('/Users/zaugg/svn.judith/judith/00_LOG/9999_MYPACKAGES/01_Nucleosomes/AnalyzeGenomicPipeline/R/overlapFeaturesAnnotations.R')
#load('data/htz1GR.rda')
#load('data/tssT.rda')
#overlapFeaturesAnnotations(annoTab=tssT,featureTab=htz1GR,align='start',up=350,down=50,projectfolder='../testACP/',featureNames='htz1')
}
